// Copyright (c) 2008 by Andrey Mirtchovski <mirtchovski@gmail.com>

/* Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * ANDREY MIRTCHOVSKI AND/OR ITS SUPPLIERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */


var currentCity = null;
var fadeTimer = null;
var currentDetails = null;
var forecastDetails = null;
var windChill = null;
var buildVersion;
var availVersion;
var updateTimer = null;
var updateInterval = null;

function setTimer(time) {
	if(updateTimer) {
		clearInterval(updateTimer);
		updateTimer = null;
	}
	updateTimer = setInterval('updateHandler()', time);
}

function stopTimer() {
	if(updateTimer) {
		clearInterval(updateTimer);
		updateTimer = null;
	}
}

// lifted from weatheroffice.gc.ca
function updatecities(event) {
	var cityFieldObj = document.getElementById('locationPopup');
	var provinceFieldObj = document.getElementById('regionPopup');
	var cityList = null;
	var provincialSummary;
	var olympicSummary;
	var cityCode = ""; 
	var foundDefaultCity = 0;
	var defaultCityIndex = 0;
	var newCitySelection = document.createElement('select');
	var cities = null;
	
	if(localizedStrings['Done'] == 'Done') {
		cities = citiesEn;
	} else {
		cities = citiesFr;
	}
	
	cityFieldObj.options.length = 0; // clear options list

	cityList = cities[provinceFieldObj[provinceFieldObj.selectedIndex].value]; // grab selected English city 
	// Create city list for selected province
	for (var cityOption=0; cityOption < cityList.length; cityOption++) {
		var optionVals = cityList[cityOption].split('|');
		cityFieldObj.options[cityFieldObj.options.length] = new Option(optionVals[0], optionVals[1]);
	}

	cityFieldObj.selectedIndex = 0;
}

//
// Function: load()
// Called by HTML body element's onload event when the widget is ready to start
//
function load()
{
    setupParts();
	
    var region = document.getElementById("regionPopup");
	var locationPopup = document.getElementById("locationPopup");
	var location = document.getElementById("location");
	var bb = document.getElementById("boxBottom");
	var ds = document.getElementById("detailsScroll");
	var sl = document.getElementById("stackLayout");
	var ui = document.getElementById("updatePopup");
	
	sl.object.setCurrentView(bb, 0);

	if(localizedStrings['Done'] == 'Done') {
		region.innerHTML = canadaRegionsEn;
		locationPopup.innerHTML = '<option value="--">Select a Location</option>';
	} else {
		region.innerHTML = canadaRegionsFr;
		locationPopup.innerHTML = '<option value="--">Choisissez un endroit</option>';	
	}
	
	var img = document.getElementById("currentImg");
	img.style.opacity = 0.0;
	img = document.getElementById("day0Img");
	img.style.opacity = 0.0;
	img = document.getElementById("day1Img");
	img.style.opacity = 0.0;
	img = document.getElementById("day2Img");
	img.style.opacity = 0.0;
	img = document.getElementById("day3Img");
	img.style.opacity = 0.0;
	img = document.getElementById("day4Img");
	img.style.opacity = 0.0;
	
	//load up the user's preferences
	if(window.widget) {
		var u = widget.preferenceForKey(createInstancePreferenceKey("units"));
        if (u == null) 
            u = widget.preferenceForKey("units");
		if (u != null) {
			units = u;
		}
        
        var p = widget.preferenceForKey(createInstancePreferenceKey("selectedRegion"));
        if(p == null)
            p = widget.preferenceForKey("selectedRegion");
		if (p != null) {
			region.selectedIndex = p;
			updatecities();
		}
        
		var c = widget.preferenceForKey(createInstancePreferenceKey("selectedLocation"));
        if(c == null) 
            c = widget.preferenceForKey("selectedLocation");
		if(c != null) {
			locationPopup.selectedIndex = c;
			location.innerHTML = locationPopup.options[c].innerHTML;
			currentCity = locationPopup.options[c].value.toLowerCase();
			//refresh();
		} else {
			if(localizedStrings['Done'] == 'Done') {
				location.innerText = "Select a location";
			} else {
				location.innerText = "Choisissez une région";	
			}
		}
		var z = widget.preferenceForKey(createInstancePreferenceKey("updateSelection"));
        if(z == null)
            z = widget.preferenceForKey("updateSelection");            
		if(z != null) {
			ui.selectedIndex = z;
		} else {
			ui.selectedIndex = 0;
		}
		updateInterval = parseInt(ui.options[ui.selectedIndex].value) * 60000;
		setTimer(updateInterval);
	}
}

//
// Function: remove()
// Called when the widget has been removed from the Dashboard
//
function remove()
{
	stopTimer();
}

//
// Function: hide()
// Called when the widget has been hidden
//
function hide()
{
    // Stop any timers to prevent CPU usage
	stopTimer();
}

//
// Function: show()
// Called when the widget has been shown
//
function show()
{
	refresh();
	setTimer(updateInterval);
}

//
// Function: sync()
// Called when the widget has been synchronized with .Mac
//
function sync()
{
}

//
// Function: showBack(event)
// Called when the info button is clicked to show the back of the widget
//
// event: onClick event from the info button
//
function showBack(event)
{
    var front = document.getElementById("front");
    var back = document.getElementById("back");

    if (window.widget) {
        widget.prepareForTransition("ToBack");
    }

    front.style.display = "none";
    back.style.display = "block";

	getVersion();
		
	if (window.widget) {
        setTimeout('widget.performTransition();', 0);
    }
}

function cleanFront()
{
    /* clean up everything */
    var cl = document.getElementById('location');
    if(localizedStrings['Done'] == 'Done') {
        cl.innerText = "Select a location";
    } else {
        cl.innerText = "Choisissez une région";	
    }
    var ct = document.getElementById('currentTemp');
    ct.innerText = '';
    var tim = document.getElementById('time');
    tim.innerText = '';
    var cw = document.getElementById('currentWind');
    cw.innerText = '';
    var fl = document.getElementById('feelsLike');
    fl.innerText = '';
    var ci = document.getElementById('currentImg');
    ci.style.opacity = 0.0;
    for(var day = 0; day < 7; day++) {
        var cimg = document.getElementById('day'+day+'Img');
        cimg.style.opacity = 0.0;
        var chigh = document.getElementById('day'+day+'High');
        chigh.innerText = '';
        var clow = document.getElementById('day'+day+'Low');
        clow.innerText = '';
        var cname = document.getElementById('day'+day);
        cname.innerText = '';
    }
}

//
// Function: showFront(event)
// Called when the done button is clicked from the back of the widget
//
// event: onClick event from the done button
//
function showFront(event)
{
    var front = document.getElementById("front");
    var back = document.getElementById("back");
	var city = document.getElementById("locationPopup");
	var region = document.getElementById("regionPopup");
	var ui = document.getElementById("updatePopup");
    var unts = document.getElementById("units");
    var defaultlocation = document.getElementById("defaultlocation");
    if(unts.checked) {
        units = 'i';
    } else {
        units = 'm';
    }
	
	if(region.options[region.selectedIndex].value != "--") {
		var loc = document.getElementById("location");
		
		loc.innerHTML = city.options[city.selectedIndex].innerHTML;
		currentCity = city.options[city.selectedIndex].value.toLowerCase();
		
		refresh();
		if(window.widget) {
			widget.setPreferenceForKey(city.selectedIndex, createInstancePreferenceKey("selectedLocation"));
			widget.setPreferenceForKey(region.selectedIndex, createInstancePreferenceKey("selectedRegion"));
			widget.setPreferenceForKey(ui.selectedIndex, createInstancePreferenceKey("updateSelection"));
			widget.setPreferenceForKey(units, createInstancePreferenceKey("units"));
            if(defaultlocation.checked) {
                widget.setPreferenceForKey(city.selectedIndex, "selectedLocation");
                widget.setPreferenceForKey(region.selectedIndex, "selectedRegion");
                widget.setPreferenceForKey(ui.selectedIndex, "updateSelection");
                widget.setPreferenceForKey(units, "units");
            }
		}
	} else {
        cleanFront();
    }
    if (window.widget) {
        widget.prepareForTransition("ToFront");
    }

    front.style.display="block";
    back.style.display="none";

    if (window.widget) {
        setTimeout('widget.performTransition();', 0);
    }
}

function getLanguage()
{
	if(localizedStrings['Done'] == 'Done') {
		return 'e';
	} else {
		return 'f';
	}
}

function visitCopyrightPage(event)
{
	var lang;
	if(getLanguage() == 'e') {
		lang = 'En';
	} else {
		lang = 'Fr';
	}
	var websiteURL = "http://www.ec.gc.ca/default.asp?lang="+lang+"&xml=5830C36B-1773-4E3E-AF8C-B21F54633E0A";
	widget.openURL(websiteURL);
}

function emailAuthor(event)
{
	var websiteURL = "mailto:mirtchovski@gmail.com?subject=Weather%20Office%20Widget";
	// Show website code
	widget.openURL(websiteURL);
}


function emailMouseOver(event)
{
	var text = document.getElementById("email");
	text.style.textDecoration = 'underline';
}

function emailMouseOut(event)
{
	var text = document.getElementById("email");
	text.style.textDecoration = 'none';
}

function visitWO(event)
{
	var lang;
	var websiteURL = "http://www.weatheroffice.gc.ca";
	
	if(getLanguage() == 'f') {
		websiteURL = "http://www.meteo.gc.ca";
	}
	// Show website code
	widget.openURL(websiteURL);
}

function linkMouseOver(event)
{
	var text = document.getElementById("linkWO");
	text.style.textDecoration = 'underline';
}

function linkMouseOut(event)
{
	var text = document.getElementById("linkWO");
	text.style.textDecoration = 'none';
}

function visitForecast(event)
{
	var city = document.getElementById("locationPopup");
	var region = document.getElementById("regionPopup");
	var lang = getLanguage();
	var websiteURL = "http://www.weatheroffice.gc.ca/";
	
	if(lang == 'f') {
		websiteURL = "http://www.meteo.gc.ca/";
	}
	
	if(region.selectedIndex > 0) {
		websiteURL += 'forecast/city_'+lang+'.html?' + city.options[city.selectedIndex].value.toLowerCase() + '&unit=' + units;
		// Show website code
		widget.openURL(websiteURL);
	} else {
		showBack(null);
	}

}

function cityMouseOver(event)
{
	var text = document.getElementById("location");
	text.style.textDecoration = 'underline';
}

function cityMouseOut(event)
{
	var text = document.getElementById("location");
	text.style.textDecoration = 'none';
}

function doCity(event)
{

}

function doRegion(event)
{
	updatecities();
}

function getDOMfromXML(xml) {
	var d = document.createElement('div');
	xml = xml.substring(xml.indexOf('<body') + xml.substring(xml.indexOf('<body')).indexOf('>')+1);
	xml = xml.substring(0, xml.indexOf('</body>'));
	d.innerHTML = xml;
	
	return d;
}


function refresh() {
	var er;
	var lang = getLanguage();
	if(currentCity == null) {
		return;
	}
	SimpleAJAXCall(
		'http://www.weatheroffice.gc.ca/forecast/city_'+lang+'.html?' + currentCity + '&unit=' + units,
		function(xml) {
			// convert the HTML to an XML DOM object
			xml = getDOMfromXML(xml);
			var ct = document.getElementById('currentTemp');
            ct.innerHTML = '';
			var cw = document.getElementById('currentWind');
            cw.innerHTML = '';
			var fl = document.getElementById('feelsLike');
			var ci = document.getElementById('currentImg');
			var loc = document.getElementById('location');
			var tim = document.getElementById('time');
			var sun = document.getElementById('sunrise');
			
			var nl = xml.getElementsByTagName('div');
			var total = 0;
			var onhand = 0;
			var hasWarning = 0;
            var hasWatch = 0;
			
			for(var i = 0; i < nl.length; i++) {
				if(nl[i].id == 'warning') {
					hasWarning = 1;
				} else if(nl[i].id == 'watch') {
					hasWatch = 1;   /* should be mutually exclusive with the above */
				} else if(nl[i].id == 'currentcond') {
					var img = nl[i].getElementsByTagName('img');
					if(img.length == 1) {
						ci.src = "http://www.weatheroffice.gc.ca" + img[0].getAttribute('src');
						ci.style.opacity = 1.0;
					} else {
						ci.style.opacity = 0.0;
					}
					var temp = nl[i].getElementsByTagName('p');
					for(var j = 0; j < temp.length; j++) {
						if(temp[j].className == 'temperature') {
							ct.innerHTML = temp[j].innerHTML.replace(/[ FC\n\r\t]/g, '');
							//ct.innerHTML = ct.innerHTML.replace(/<sup>.+units.>/, '');
							ct.innerHTML = ct.innerHTML.replace(/<sup>[^>]+>/, '');
						}
					}
					var cond = nl[i].getElementsByTagName('div');
					for(var j = 0; j < cond.length; j++) {
						if(cond[j].id == 'currentcond-content') {
							currentDetails = cond[j].innerHTML;
                            currentDetails = currentDetails.replace(/<a href[^>]+>/g, '');  // webkit now attempts to load a page inside the widget. very bad
                            currentDetails = currentDetails.replace(/<\/a>/g, '');                            
						} else if(cond[j].id == 'cityobserved') {
                            observationTime = cond[j].innerHTML;
                            observationTime = observationTime.slice(observationTime.indexOf("Date:"), observationTime.length);
                            observationTime = observationTime.slice(observationTime.indexOf("<dd>"), observationTime.indexOf("</dd>"));
                            observationTime = observationTime.slice(observationTime.indexOf(">")+1, observationTime.indexOf(">")+8);
                            tim.innerHTML = "@"+observationTime;
                        }
					}
					
					var idx = nl[i].innerHTML.indexOf("Humidex");
                    if(idx < 0) {
                        if(getLanguage() == 'e') {
                            idx = nl[i].innerHTML.indexOf("Wind Chill");
                        } else {
                            idx = nl[i].innerHTML.indexOf("olien:");
                        }
                        fl.style.color = '#33DDFF';
                    } else {
                        fl.style.color = '#DD5555';
                    }
					if(idx > 0) {
						// try and get the windchill
						var wc = nl[i].innerHTML.slice(idx, nl[i].innerHTML.length);								
						wc = wc.toLowerCase();
						wc = wc.slice(wc.indexOf("<dd>"), wc.length);
						wc = wc.replace(/[ \n\r\t]/g, '');
						wc = wc.replace(/.*<dd>/, '');
						wc = wc.replace(/<\/dd>/, '');
						wc = wc.replace(/<ddclass.*/, '');
						fl.innerHTML = wc + "&deg;";
					} else {
						fl.innerHTML = '';
					}
					
					var aidx;
					if(getLanguage() == 'e') {
						aidx = nl[i].innerHTML.indexOf("Air Quality");
					} else {
						aidx = nl[i].innerHTML.indexOf("Cote air");
					}
					if(aidx > 0) {
						// try and get the air quality
						var wc = nl[i].innerHTML.slice(aidx, nl[i].innerHTML.length);								
						wc = wc.toLowerCase();
						wc = wc.slice(wc.indexOf(":"), wc.length);
						wc = wc.slice(wc.indexOf("<dd>"), wc.indexOf("</dd>"));
						wc = wc.replace(/[ \n\r\t]/g, '');
						wc = wc.replace(/<dd>/, '');
						wc = wc.replace(/<\/dd>/, '');
						var x = parseInt(wc);
						if(x > 10)
							x = 11;
						ct.style.color = aircolors[x];
					} else {
						ct.style.color = '#FFFFFF';
					}
					var widx;
					if(getLanguage() == 'e') {
						widx = nl[i].innerHTML.indexOf("Wind");
					} else {
						widx = nl[i].innerHTML.indexOf("Vent");
					}
					if(widx > 0) {
						// try and get the air quality
						var wc = nl[i].innerHTML.slice(widx, nl[i].innerHTML.length);								
						//wc = wc.toLowerCase();
						wc = wc.slice(wc.indexOf(":"), wc.length);
						wc = wc.slice(wc.indexOf("<dd"), wc.indexOf("</dd>"));
                        wc = wc.slice(wc.indexOf(">")+1, wc.length);
						wc = wc.replace(/[\n\r\t]/g, '');
						cw.innerHTML = wc;
					} else {
                        cw.innerHTML = '';
                    }
                } else if(nl[i].id == 'historicaldata') {
					var sunx;
                    var suny;
					if(getLanguage() == 'e') {
						sunx = nl[i].innerHTML.indexOf("Sunrise");
						suny = nl[i].innerHTML.indexOf("Sunset");
					} else {
						sunx = nl[i].innerHTML.indexOf("Lever");
						suny = nl[i].innerHTML.indexOf("Coucher");
					}
					if(sunx > 0 && suny > 0) {
						// try and get the sunrise/sunset
						var rise = nl[i].innerHTML.slice(sunx, suny);
                        var set = nl[i].innerHTML.slice(suny, suny+50);						
                        rise = rise.slice(rise.indexOf('<dd>')+4, rise.indexOf('</dd>'))
                        set = set.slice(set.indexOf('<dd>')+4, set.indexOf('</dd>'))
						sun.innerHTML = rise +" ⟷ "+ set;
					} else {
                        sun.innerHTML = '';
                    }
				} else if(nl[i].id == 'cityf') {
					var periods = nl[i].getElementsByTagName('div');
					var day = 0;
					for(j = 0; j < periods.length; j++) {
						if(periods[j].className == 'fperiod' || periods[j].className == 'fperiod fperiodlast') {
							var title = periods[j].getElementsByTagName('h3');
							var cimg = document.getElementById('day'+day+'Img');
							var chigh = document.getElementById('day'+day+'High');
							var clow = document.getElementById('day'+day+'Low');
							var cpop = document.getElementById('day'+day+'Pop');
							var cname = document.getElementById('day'+day);
							if(title.length == 1) {
								cname.innerHTML = title[0].innerHTML;//.substring(0,3);
							} else {
								cname = "";
							}
							var img = periods[j].getElementsByTagName('img');
							if(img.length == 1) {
								cimg.src = "http://www.weatheroffice.gc.ca" + img[0].getAttribute('src');
								cimg.style.opacity = 1.0;
							} else {
								cimg.style.opacity = 0.0;
							}
							var high = periods[j].getElementsByTagName('li');
							for(k = 0; k < high.length; k++) {
								var todelete = high[k];
								if(high[k].className == 'high') {
									chigh.innerHTML = high[k].innerHTML.replace(/[ \n\r\tFC]/g, '');
								} else if(high[k].className == 'low') {
									clow.innerHTML = high[k].innerHTML.replace(/[ \n\r\tFC]/g, '');	
								} else if(high[k].className == 'pop') {
									cpop.innerHTML = high[k].innerHTML.replace(/[ \n\r\t]/g, '');
								}
							}
							day++;
						} else if(periods[j].className == 'fdetails') {
                            det = periods[j].getElementsByTagName('div');
                            forecastDetails = '';
                            for(z = 0; z < det.length; z++) {
                                forecastDetails += det[z].innerHTML;
                            }
						}
					}
					var sl = document.getElementById('detailsScroll');
					sl.object.content.innerHTML = currentDetails + forecastDetails;
//					sl.object.refresh();
				}
			}
			
			/* deal with warnings */
			if(hasWarning) {
				loc.style.color = '#FF0000';
			} else if(hasWatch) {
                loc.style.color = '#FFFF00';
            } else {
				loc.style.color = '#DDDDDD';
			}
		},
		'GET',
        null
    );
}

function detOver(event)
{
	var scr = document.getElementById("detailsScroll");
	var sl = document.getElementById("stackLayout");
		
	sl.object.setCurrentView(scr, 0);
	scr.object.refresh();
}

function detOut(event)
{
	var bottom = document.getElementById("boxBottom");
	var sl = document.getElementById("stackLayout");

	sl.object.setCurrentView(bottom, 1);
}

function windChillOver(event)
{
    var wc = document.getElementById("feelsLike");
	if(wc.innerHTML == '') {
		windChill = '';
		return;
	}
	windChill = wc.innerHTML;
	wc.innerHTML = localizedStrings['Feels Like'];
}

function windChillOut(event)
{
	if(windChill == '')
		return;
    var wc = document.getElementById("feelsLike");
	if(windChill) {
		wc.innerHTML = windChill;
	}
}

function visitVers(event)
{
	var websiteURL = "http://www.mirtchovski.com/code/environment.html";
	widget.openURL(websiteURL);
}

function versionMouseOut(event)
{
	var text = document.getElementById("newVersion");
	text.style.textDecoration = 'none';
}

function versiouMouseOver(event)
{
	var text = document.getElementById("newVersion");
	text.style.textDecoration = 'underline';
}

function setUpdatePrefs(event)
{
	var ui = document.getElementById('updatePopup');
	
	updateInterval = parseInt(ui.options[ui.selectedIndex].value*60000); // value is given in minutes
	setTimer(updateInterval); 
}

function updateHandler()
{
	refresh();
}

if (window.widget) {
    widget.onremove = remove;
    widget.onhide = hide;
    widget.onshow = show;
    widget.onsync = sync;
}


function donation(event)
{
    url = "https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=E2KEFZ3CB4VLQ&lc=CA&item_name=mirtchovski%2ecom%20dashboard%20widgets&item_number=Weather%20Office%20%26%20Environment%20Canada%20Satellite%20Images&currency_code=CAD&bn=PP%2dDonationsBF%3abtn_donate_SM%2egif%3aNonHosted";
    
    widget.openURL(url);
}
